/*
 * Message.java
 *
 * Created on 11 de Novembro de 2001, 18:40
 */

package br.shmem;
import java.io.Serializable;
import java.net.URL;

/** represents all the messages exchanged by hosts.
 *
 * @author crolmos & tminc
 */
public class Message implements Serializable {
    
    public static final int LOGIN = 1;
    public static final int LOGOUT = 2;
    public static final int SHARE = 3;
    public static final int UNSHARE = 4;
    public static final int VALIDATE = 5;
    public static final int REQUEST = 6;
    public static final int BEGIN_TRANS = 7;
    public static final int COMMIT = 8;
    public static final int ROLLBACK = 9;
    public static final int PING = 10;
    public static final int GET = 11;
    public static final int RELOAD = 12;
    public static final int UNLOCK = 13;
    public static final int IDENTIFY = 14;
    public static final int UPDATE = 15;
    
    private int type;
    private String globalName;
    private boolean error = false;
    private TimeStamp timeStamp = null;
    private HostId hostId = null;
    private SharedObject data = null;
    private String errorDescription = null;
    private URL url = null;

    /** accesor for the type of the message.
     * A valid type is one of the static constants defined above.
     */   
    public int getType() { return type; }

    /** accesor for the global name of the object.
     */    
    public String getGlobalName() { return globalName; }

    /** <CODE>true</CODE> if the message indicates an error, <CODE>false</CODE> otherwise.
     */
    public boolean isError() { return error; }

    /** accesor for the time stamp of the object.
     *@see br.shmem.TimeStamp
     */    
    public TimeStamp getTimeStamp() { return timeStamp; }

    /** accesor for the host id of the object.
     *@see br.shmem.HostId
     */    
    public HostId getHostId() { return hostId; }

    /** accesor for the data of the object.
     *@see java.io.Serializable
     */    
    public SharedObject getData() { return data; }

    /** accesor for the error description of the object.
     */    
    public String getErrorDescription() { return errorDescription; }
    
    /** accesor for the url of the object.
     */    
    public URL getURL() { return url; }
    
    /** Creates new Message */
    private Message() {
    }

    /** creates a message of this type.
     * @param hostId
     * @return
     */    
    public static Message createLogin(HostId hostId) {
        Message m = new Message();
        m.type = LOGIN;
        m.hostId = hostId;
        return m;
    }
    
    /** creates a message of this type.
     */    
    public static Message createLogout() {
        Message m = new Message();
        m.type = LOGOUT;
        return m;
    }
    
    /** creates a message of this type.
     * @param globalName the name of the object in the global space.
     */    
    public static Message createShare(String globalName) {
        Message m = new Message();
        m.type = SHARE;
        m.globalName = globalName;
        return m;
    }
    
    /** creates a message of this type.
     * @param globalName the name of the object in the global space.
     */    
    public static Message createUnshare(String globalName) {
        Message m = new Message();
        m.type = UNSHARE;
        m.globalName = globalName;
        return m;
    }
    
    /** creates a message of this type.
     * @param globalName the name of the object in the global space.
     * @param timeStamp the timestamp of the object to be validated.
     */    
    public static Message createValidate(String globalName, TimeStamp timeStamp) {
        Message m = new Message();
        m.type = VALIDATE;
        m.globalName = globalName;
        m.timeStamp = timeStamp;
        return m;
    }
    
    /** creates a message of this type.
     * @param globalName the name of the object in the global space.
     */    
    public static Message createRequest(String globalName) {
        Message m = new Message();
        m.type = REQUEST;
        m.globalName = globalName;
        return m;
    }
    
    /** creates a message of this type.
     * @param globalName the name of the object in the global space.
     */    
    public static Message createBeginTrans(String globalName) {
        Message m = new Message();
        m.type = BEGIN_TRANS;
        m.globalName = globalName;
        return m;
    }
    
    /** creates a message of this type.
     * @param globalName the name of the object in the global space.
     */    
    public static Message createCommit(String globalName) {
        Message m = new Message();
        m.type = COMMIT;
        m.globalName = globalName;
        return m;
    }
    
    /** creates a message of this type.
     * @param globalName the name of the object in the global space.
     */    
    public static Message createRollback(String globalName) {
        Message m = new Message();
        m.type = ROLLBACK;
        m.globalName = globalName;
        return m;
    }
    
    /** creates a message of this type.
     */    
    public static Message createPing() {
        Message m = new Message();
        m.type = PING;
        return m;
    }
    
    /** creates a message of this type.
     * @param globalName the name of the object in the global space.
     * @param hostId the remote host's id.
     */    
    public static Message createGet(String globalName, HostId hostId) {
        Message m = new Message();
        m.type = GET;
        m.globalName = globalName;
        m.hostId = hostId;
        return m; 
    }

    /** creates a message of this type.
     * @param globalName the name of the object in the global space.
     * @param hostId the remote host's id.
     */    
    public static Message createReload(String globalName, HostId hostId) {
        Message m = new Message();
        m.type = RELOAD;
        m.globalName = globalName;
        m.hostId = hostId;
        return m; 
    }
   
    /** creates a message of this type.
     * @param globalName the name of the object in the global space.
     * @param timeStamp the timestamp of the object.
     */    
    public static Message createUnlock(String globalName, HostId writer, TimeStamp timeStamp) {
        Message m = new Message();
        m.type = UNLOCK;
        m.globalName = globalName;
        m.timeStamp = timeStamp;
        m.hostId = writer;
        return m;
    }
    
    /** creates a message of this type.
     * @param globalName the name of the object in the global space.
     * @param timeStamp the timestamp of the object.
     */    
    public static Message createIdentify(URL url) {
        Message m = new Message();
        m.type = IDENTIFY;
        m.url = url;
        return m;
    }
    
    /** creates a message of this type.
     * @param globalName the name of the object in the global space.
     * @param hostId the remote host's id.
     */    
    public static Message createUpdate(String globalName, HostId hostId) {
        Message m = new Message();
        m.type = UPDATE;
        m.globalName = globalName;
        m.hostId = hostId;
        return m; 
    }
   
     /** creates a message of this type.
     * @param globalName the name of the object in the global space.
     * @param data the serializable object to be sent.
     */    
    public static Message createOK(int type, String globalName, SharedObject data) {
        Message m = new Message();
        m.type = type;
        m.globalName = globalName;
        m.data = data;
        return m;
    }
   
    
    /** creates a message of this type.
     * @param type one of the static constants defined above.
     */    
    public static Message createOK(int type) {
        Message m = new Message();
        m.type = type;
        return m;
    }

    /** creates a message of this type.
     * @param type one of the static constants defined above.
     * @param hostId the remote host's id.
     */    
    public static Message createOK(int type, HostId hostId) {
        Message m = new Message();
        m.type = type;
        m.hostId = hostId;
        return m;
    }
    
    /** creates a message of this type.
     * @param type one of the static constants defined above.
     * @param hostId the remote host's id.
     * @param timeStamp the timestamp of the object.
     */    
    public static Message createOK(int type, HostId hostId, TimeStamp timeStamp) {
        Message m = new Message();
        m.type = type;
        m.hostId = hostId;
        m.timeStamp = timeStamp;
        return m;
    }
    
    /** creates a message of this type.
     * @param type one of the static constants defined above.
     * @param timeStamp the timestamp of the object.
     */    
    public static Message createOK(int type, TimeStamp timeStamp) {
        Message m = new Message();
        m.type = type;
        m.timeStamp = timeStamp;
        return m;
    }

    /** creates a message of this type.
     * @param type one of the static constants defined above.
     */    
    public static Message createError(int type, String errorDescription) {
        Message m = new Message();
        m.type = type;
        m.error = true;
        m.errorDescription = errorDescription;
        return m;
    }

    /** creates a message of this type.
     * @param type one of the static constants defined above.
     * @param hostId the remote host's id.
     */    
    public static Message createError(int type, HostId hostId, String errorDescription) {
        Message m = new Message();
        m.type = type;
        m.hostId = hostId;
        m.error = true; 
        m.errorDescription = errorDescription;
        return m;
    }
    
    
}
