/*
 * SharedReference.java
 *
 * Created on 4 de Dezembro de 2001, 22:48
 */

package br.shmem;
import java.lang.reflect.*;

/** This class represents a reference to shared object.
 * <strong>WARNING:</strong> SharedReferences must not be shared locally. Get many of them from the <code>SharedMemory</code> instead!
 * @author crolmos & tminc
 */
public class SharedReference {

    public static final long defaultLeaseTime = 20000;

    private String globalName;
    private SharedMemory shmem;
    
    /** Creates new SharedReference */
    private SharedReference() {
    }
    
    /** Creates new SharedReference.<br>
     * This constructor should never be called directly. Instead use {@link br.shmem.SharedMemory.get(String) br.shmem.SharedMemory.get}.
     * @param shmem the <code>SharedMemory</code> in wich this object is shared.
     * @param globalName the global name of the object that will be accessed throw this reference.
     */    
    public SharedReference(SharedMemory shmem, String globalName) {
        this.shmem = shmem;
        this.globalName = globalName;
    }
    
    /** Acessor for the global name.
     * @return the global name of the shared object.
     */    
    public String getGlobalName() {
        return globalName;
    }

    private Object getData() {
	return shmem.getCachedObject(globalName).getData();
    }
       
    

    /** This method calls a method from the shared object.<br>
     * If the lease has expired it validates the object before the call is made.
     * @param name the name of the method.
     * @param argTypes the types of the arguments.
     * @param args the arguments.
     */
    public Object callMethod(String name, Class argTypes[], Object args[]) 
	throws InvocationException, CommLayerException, InsecureObjectException, InvalidOperationException
    {
        SharedObject sharedObject = shmem.getCachedObject(globalName);
        if(!sharedObject.isUpdated()) 
            validate();
	try {
	    Object obj = getData();
	    Class rc = obj.getClass();
	    Method m = rc.getMethod(name, argTypes);
	    return m.invoke(obj, args);
	}
	catch (NoSuchMethodException e) {
	    throw new InvocationException(e.fillInStackTrace());
	}
	catch (IllegalAccessException e) {
	    throw new InvocationException(e.fillInStackTrace());
	}
	catch (IllegalArgumentException e) {
	    throw new InvocationException(e.fillInStackTrace());
	}
	catch (InvocationTargetException e) {
	    throw new InvocationException(e.fillInStackTrace());
	}
    }

    /** Unshares this object from the global space.
    */    
    public void unshare()
    throws CommLayerException, InsecureObjectException, InvalidOperationException
    {
        shmem.unshare(this);
    }
     
    /** Validates this object.
     */    
    public void validate()
    throws CommLayerException, InsecureObjectException, InvalidOperationException
    {
        shmem.validate(this);
    }
    
    /** Initiates a transaction for this object.
     */    
    public void beginTrans()
    throws CommLayerException, InsecureObjectException, InvalidOperationException
    {
        shmem.beginTrans(this);
    }
    
    /** Commits a transaction.
     */    
    public void commit()
    throws CommLayerException, InsecureObjectException, InvalidOperationException
    {
        shmem.commit(this);
    }
    
    /** Rolls back a transaction.
     */    
    public void rollback()
    throws CommLayerException, InsecureObjectException, InvalidOperationException
    {
        shmem.rollback(this);
    }
}





