/*
 * SharedMemory.java
 *
 * Created on 4 de Dezembro de 2001, 20:32
 */
package br.shmem.server;

import br.shmem.*;
import java.security.*;
import java.util.*;
import java.net.*;
import java.io.*;

/** This class represents a global space.
 * @author  crolmos & tminc
 */
public class Server {

    public static String configurationFile = "server.ini";
    
    private Hashtable hosts; // URLs and HostIds;
    private Hashtable objects;
    
    private PrivateKey pKey;
    private HostId myId;
    private URL plainSocketURL;    
    
    private boolean debug = false;
    
    private ServerCommLayer commLayer;
    
    /** Creates new Server
     * @param hostname the host where this server will be listening.
     * @param plainPort the port in wich this server will be listening for logins.
     * @param signedPort the port in wich this server will be listening for other operations.
     */
    public Server(String hostname, int plainPort, int signedPort)
    throws InsecureObjectException, CommLayerException {
        try {
            System.out.println("Starting Server...");
            hosts = new Hashtable();
            objects = new Hashtable();
            KeyPair keys = generateKeyPair();
            pKey = keys.getPrivate();
            myId = new HostId(new URL("http://" + hostname + ":" + signedPort), keys.getPublic());
            plainSocketURL = new URL("http://" + hostname + ":" + plainPort);
            commLayer = new ServerCommLayer(this);
            commLayer.start();
            System.out.println("Server Running.");
            System.out.println("Hostname:         " + hostname);
            System.out.println("Plain port:  " + plainPort);
            System.out.println("Signed port: " + signedPort);
        }
        catch(MalformedURLException e) {
            throw new CommLayerException(e.toString());
        }
    }
    
    protected KeyPair generateKeyPair()
    throws InsecureObjectException {
        try {
            KeyPairGenerator generator = KeyPairGenerator.getInstance(ServerCommLayer.keyPairGeneratorAlgorithm);
            return generator.generateKeyPair();
        }
        catch(NoSuchAlgorithmException e) {
            throw new InsecureObjectException(e.toString());
        }
    }
    
    public void setDebugging(boolean flag) {
        debug = flag;
    }
    
    public boolean isDebugging() {
        return debug;
    }
    
    public ServerCommLayer getCommLayer() {
        return commLayer;
    }
    
    public PrivateKey getPrivateKey() {
        return pKey;
    }
    
    public URL getPlainSocketURL() {
        return plainSocketURL;
    }
    
    public HostId getHostId() {
        return myId;
    }
    
    /** This method returns a list of all the clients logged in.
     */    
    public List getClients() {
        return new ArrayList(hosts.keySet());
    }
    
    /** This method returns a list of all the objects shared in this global space.
     */    
    public List getObjects() {
        ArrayList list = new ArrayList();
        Enumeration en = objects.keys();
        while(en.hasMoreElements()) {
            String globalName = (String)en.nextElement();
            SharedEntry entry = (SharedEntry)objects.get(globalName);
            if(entry != null)
                list.add(new ObjectInfo(globalName, entry));
        }
        return list;
    }
    
    public HostId getHostId(URL clientURL)
    throws HostNotFoundException {
        HostId hostId = (HostId)hosts.get(clientURL);
        if(hostId == null)
            throw new HostNotFoundException("Host not found: " + clientURL);
        return hostId;
    }
    
    protected boolean login(HostId client) {
        synchronized(hosts) {
            if(!hosts.containsKey(client.getURL())) {
                hosts.put(client.getURL(), client);
                return true;
            }
            return false;
        }
    }
    
    protected boolean logout(URL client) {
        synchronized(hosts) {
            synchronized(objects) {
                if (hosts.remove(client) != null) {
                    Enumeration keys = objects.keys();
                    while(keys.hasMoreElements()) {
                        Object key = keys.nextElement();
                        SharedEntry sharedEntry = (SharedEntry)objects.get(key);
                        if(sharedEntry.getOwner().equals(client))
                            objects.remove(key);
                    }
                    return true;
                }
                return false;
            }
        }
    }
    
    protected boolean ping(URL client) {
        return commLayer.ping(client);
    }
    
    protected TimeStamp share(URL client, String globalName, boolean lockable)
    throws AlreadySharedException, InsecureObjectException {
        synchronized(objects) {
            if (objects.containsKey(globalName))
                throw new AlreadySharedException(globalName + " is already shared");
            SharedEntry entry = new SharedEntry(client, lockable);
            TimeStamp ts = new TimeStamp(new Date(), pKey);
            entry.setTimeStamp(ts);
            objects.put(globalName, entry);
            return ts;
        }
    }
    
    protected void unshare(URL client, String globalName)
    throws NotSharedException, InvalidOperationException {
        
        try {
            synchronized(objects) {
                SharedEntry entry = (SharedEntry)objects.get(globalName);
                if (entry == null) {
                    throw new NotSharedException(globalName + " is not shared");
                }
                if (entry.getOwner().equals(client)) {
                    if (entry.isLocked()) {
                        entry.setLockable(false);
                        while (entry.isLocked()) objects.wait();
                    }
                    if (objects.containsKey(globalName))
                        objects.remove(globalName);
                    else
                        throw new NotSharedException(globalName + " is not shared");
                }
                else
                    throw new InvalidOperationException(client + " is not the owner of " + globalName);
                
            }
        }
        catch (InterruptedException ie) {
        }
    }
    
    protected HostId request(String globalName)
    throws NotSharedException {
        
        synchronized(hosts) {
            synchronized(objects) {
                SharedEntry entry = (SharedEntry)objects.get(globalName);
                if (entry == null)
                    throw new NotSharedException(globalName + " is not shared");
                return (HostId) hosts.get(entry.getOwner());
            }
        }
        
    }
    
    protected HostId validate(String globalName, TimeStamp stamp)
    throws NotSharedException, InsecureObjectException {
        synchronized(hosts) {
            synchronized (objects) {
                SharedEntry entry = (SharedEntry)objects.get(globalName);
                if (entry == null)
                    throw new NotSharedException(globalName + " is not shared");
                if (entry.getTimeStamp().getDate().compareTo(stamp.getDate()) > 0)
                    return (HostId) hosts.get(entry.getOwner());
                else
                    return null;
            }
        }
    }
    
    protected TimeStamp beginTrans(URL client, String globalName)
    throws NotSharedException, AlreadyLockedException, ObjectNotLockableException {
        synchronized(objects) {
            SharedEntry entry = (SharedEntry)objects.get(globalName);
            if (entry == null)
                throw new NotSharedException(globalName + " is not shared");
            entry.lock(client);
            return entry.getTimeStamp();
        }
    }
    
    protected TimeStamp commit(URL client, String globalName)
    throws NotSharedException, InvalidOperationException, InsecureObjectException {
        synchronized(objects) {
            SharedEntry entry = (SharedEntry)objects.get(globalName);
            if (entry == null)
                throw new NotSharedException(globalName + " is not shared");
            if (entry.getWriter() == null || !entry.getWriter().equals(client))
                throw new InvalidOperationException(client + " is not the writer of " + globalName);
            return new TimeStamp(new Date(), pKey);
        }
    }
    
    protected void rollback(URL client, String globalName)
    throws NotSharedException, NotLockedException {
        synchronized(objects) {
            SharedEntry entry = (SharedEntry)objects.get(globalName);
            if (entry == null)
                throw new NotSharedException(globalName + " is not shared");
            entry.unlock(client);
            objects.notifyAll();
        }
    }
    
    protected void unlock(URL client, String globalName, TimeStamp stamp)
    throws NotSharedException, InsecureObjectException, NotLockedException {
        synchronized(objects) {
            SharedEntry entry = (SharedEntry)objects.get(globalName);
            if (entry == null)
                throw new NotSharedException(globalName + " is not shared");
            if (!stamp.verify(myId.getKey()))
                throw new InsecureObjectException("Timestamp for " + globalName + " failed signature verification");
            entry.setTimeStamp(stamp);
            entry.unlock(client);
            objects.notifyAll();
        }
    }
    
    /** For testing
     */
    public static void main(String args[]) {
        try {
            String url = null;
            int plainPort = -1;
            int signedPort = -1;
            FileInputStream inStream = new FileInputStream(configurationFile);
            Properties prop = new Properties();
            prop.load(inStream);
            url = prop.getProperty("url");
            if(url == null) {
                System.err.println("Undefined url.");
                System.exit(-1);
            }
            try {
                plainPort = Integer.parseInt(prop.getProperty("plain_port"));
            }
            catch(NumberFormatException e) {
                System.err.println("Undefined plain_port.");
                System.exit(-1);
            }
            try {
                signedPort = Integer.parseInt(prop.getProperty("signed_port"));
            }
            catch(NumberFormatException e) {
                System.err.println("Undefined signed_port.");
                System.exit(-1);
            }
            
            Server server = new Server(url, plainPort, signedPort);
            server.setDebugging(true);
        }
        catch(Exception e) {
            System.err.println(e);
        }
    }
}







